@@ -4,12 +4,6 @@
 #####################################################################
 # A set of classes implementing Blueprints API for Neo4j engine.    #
 #                                                                   #
-# The CulturePlex Laboratory                                        #
-# The University of Western Ontario                                 #
-#                                                                   #
-# Diego Muñoz Escalante (escalant3 at gmail dot com)                #
-# Javier de la Rosa (versae at gmail dot com)                       #
-#                                                                   #
 # File: pyblueprints/neo4j.py                                       #
 #####################################################################
 
@@ -104,7 +98,7 @@ class Neo4jGraph(Graph):
         raise NotImplementedError("Method has to be implemented")
 
 
-class Element():
+class Element(object):
     """An class defining an Element object composed
     by a collection of key/value properties for the
     Neo4j database"""
@@ -224,7 +218,7 @@ class Edge(Element):
                                 self.neoelement.properties)
 
 
-class Index():
+class Index(object):
     """An class containing all the methods needed by an
     Index object"""
 
@@ -285,10 +279,12 @@ class Index():
         @params value: Index value string
         @returns A generator of Vertex or Edge objects"""
         for element in self.neoindex[key][value]:
-            if self.indexClass == "VERTICES":
+            if self.indexClass == "vertex":
                 yield Vertex(element)
-            else:
+            elif self.indexClass == "edge":
                 yield Edge(element)
+            else:
+                raise TypeError(self.indexClass)
 
     def remove(self, key, value, element):
         """Removes an element from an index under a given
@@ -321,7 +317,7 @@ class Neo4jIndexableGraph(Neo4jGraph):
             index = self.neograph.relationships.indexes.create(indexName)
         else:
             NameError("Unknown Index Class %s" % indexClass)
-        return Index(indexName, indexClass, "automatic", index)
+        return Index(indexName, indexClass, "manual", index)
 
     def createAutomaticIndex(self, indexName, indexClass):
         """Creates an index automatically managed my Neo4j
@@ -364,6 +360,139 @@ class Neo4jIndexableGraph(Neo4jGraph):
             indexObject = self.neograph.relationships.indexes.get(indexName)
             yield Index(indexName, "edge", "manual", indexObject)
 
-    def dropIndex(self, indexName):
-        """TODO Documentation"""
-        raise NotImplementedError("Method has to be implemented")
+    def dropIndex(self, indexName, indexClass):
+        index = self.getIndex(indexName, indexClass)
+        index.neoindex.delete()
+
+
+class TransactionalElement(Element):
+
+    _graphContainer = None
+
+    def __init__(self, element, graph):
+        super(TransactionalElement, self).__init__(element.neoelement)
+        self._graphContainer = graph
+
+    def __transactionalOperation(self, operation, *args, **kwargs):
+        op = getattr(super(TransactionalElement, self), operation)
+        if self._graphContainer._transaction:
+            with self._graphContainer._txObj:
+                return op(*args, **kwargs)
+        else:
+            return op(*args, **kwargs)
+
+    def setProperty(self, *args, **kwargs):
+        return self.__transactionalOperation('setProperty', *args, **kwargs)
+
+    def removeProperty(self, *args, **kwargs):
+        return self.__transactionalOperation('removeProperty', *args, **kwargs)
+
+
+class TransactionalVertex(TransactionalElement, Vertex):
+
+    def __yieldEdges(self, edges):
+        for edge in edges:
+            yield TransactionalEdge(edge, self._graphContainer)
+
+    def getOutEdges(self, *args, **kwargs):
+        edges = super(TransactionalVertex, self).getOutEdges(*args, **kwargs)
+        return self.__yieldEdges(edges)
+
+    def getInEdges(self, *args, **kwargs):
+        edges = super(TransactionalVertex, self).getInEdges(*args, **kwargs)
+        return self.__yieldEdges(edges)
+
+    def getBothEdges(self, *args, **kwargs):
+        edges = super(TransactionalVertex, self).getBothEdges(*args, **kwargs)
+        return self.__yieldEdges(edges)
+
+
+class TransactionalEdge(TransactionalElement, Edge):
+
+    def getOutVertex(self, *args, **kwargs):
+        vertex = super(TransactionalEdge, self).getOutVertex(*args, **kwargs)
+        return TransactionalVertex(vertex, self._graphContainer)
+
+    def getInVertex(self, *args, **kwargs):
+        vertex = super(TransactionalEdge, self).getInVertex(*args, **kwargs)
+        return TransactionalVertex(vertex, self._graphContainer)
+
+
+class TransactionalIndex(Index):
+
+    _graphContainer = None
+
+    def __init__(self, index, graph):
+        super(TransactionalIndex, self).__init__(index.indexName,
+                                                index.indexClass,
+                                                index.indexType,
+                                                index.neoindex)
+        self._graphContainer = graph
+
+    def get(self, *args, **kwargs):
+        elements = super(TransactionalIndex, self).get(*args, **kwargs)
+        if self.indexClass == "vertex":
+            for element in elements:
+                yield TransactionalVertex(element, self._graphContainer)
+        elif self.indexClass == "edge":
+            for element in elements:
+                yield TransactionalEdge(element, self._graphContainer)
+        else:
+            raise TypeError(self.indexClass)
+
+
+class Neo4jTransactionalGraph(Neo4jGraph):
+    """An class containing the specific methods
+    for transacional graphs"""
+
+    _transaction = False
+    _txObj = None
+
+    def startTransaction(self):
+        self._txObj = self.neograph.transaction(commit=False)
+        self._transaction = True
+
+    def stopTransaction(self):
+        self._txObj.commit()
+        self._txObj = None
+        self._transaction = False
+
+    def __transactionalOperation(self, operation, *args, **kwargs):
+        op = getattr(super(Neo4jTransactionalGraph, self), operation)
+        if self._transaction:
+            with self._txObj:
+                return op(*args, **kwargs)
+        else:
+            return op(*args, **kwargs)
+
+    def addVertex(self, *args, **kwargs):
+        vertex = self.__transactionalOperation('addVertex', *args, **kwargs)
+        return TransactionalVertex(vertex, self)
+
+    def getVertex(self, *args, **kwargs):
+        vertex = self.__transactionalOperation('getVertex', *args, **kwargs)
+        return TransactionalVertex(vertex, self) if vertex else None
+    
+    def removeVertex(self, *args, **kwargs):
+        return self.__transactionalOperation('removeVertex', *args, **kwargs)
+
+    def addEdge(self, *args, **kwargs):
+        edge = self.__transactionalOperation('addEdge', *args, **kwargs)
+        return TransactionalEdge(edge, self)
+
+    def getEdge(self, *args, **kwargs):
+        return self.__transactionalOperation('getEdge', *args, **kwargs)
+
+    def removeEdge(self, *args, **kwargs):
+        return self.__transactionalOperation('removeEdge', *args, **kwargs)  
+
+
+class Neo4jTransactionalIndexableGraph(Neo4jTransactionalGraph, Neo4jIndexableGraph):
+
+    def createManualIndex(self, *args, **kwargs):
+        index = super(Neo4jTransactionalIndexableGraph, self).createManualIndex(*args, **kwargs)
+        return TransactionalIndex(index, self) if index else None
+
+    def getIndex(self, *args, **kwargs):
+        index = super(Neo4jTransactionalIndexableGraph, self).getIndex(*args, **kwargs)
+        return TransactionalIndex(index, self) if index else None
